SystemOrganization addCategory: #TusLibros!

Object subclass: #CartSession
	instanceVariableNames: 'cart id timestamp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CartSession class methodsFor: 'instance creation' stamp: 'achavez 6/8/2013 23:11'!
for: aCart id: aNumber timestamp: aTime
	^self new for: aCart id: aNumber timestamp: aTime! !

!CartSession class methodsFor: 'pirvate' stamp: 'achavez 6/9/2013 01:34'!
timeOutInSeconds
	^30*60! !

!CartSession methodsFor: 'accessing' stamp: 'achavez 6/8/2013 23:12'!
cart
	^ cart! !

!CartSession methodsFor: 'creation' stamp: 'achavez 6/9/2013 00:50'!
for: aCart id: aNumber timestamp: aTime
	cart := aCart.
	id := aNumber.
	timestamp := aTime! !

!CartSession methodsFor: 'accessing' stamp: 'achavez 6/9/2013 00:32'!
id
	^ id! !

!CartSession methodsFor: 'testing' stamp: 'achavez 6/9/2013 01:33'!
isTimedOutAt: aTime
	^(timestamp addSeconds: self class timeOutInSeconds) < aTime ! !

!CartSession methodsFor: 'accessing' stamp: 'achavez 6/8/2013 23:32'!
timestamp
	^ timestamp! !

Object subclass: #Cashier
	instanceVariableNames: 'sales priceList date merchantProcessor'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cashier class methodsFor: 'error' stamp: 'achavez 5/30/2013 20:07'!
creditCardExpired
	^'Credit card has expired'! !

!Cashier class methodsFor: 'instance creation' stamp: 'achavez 6/3/2013 20:16'!
date: aDate withPriceList: aPriceList merchantProcessor: aMerchantProcessor 
	 ^self new date: aDate withPriceList: aPriceList merchantProcessor: aMerchantProcessor ! !

!Cashier class methodsFor: 'error' stamp: 'msalegas 5/29/2013 20:23'!
invalidCheckOut
	^'Invalid check out'! !

!Cashier methodsFor: 'services' stamp: 'achavez 6/9/2013 02:40'!
checkout: aCart card: aCard 
	| price sale result |
	aCart isEmpty ifTrue: [ ^self error: self class invalidCheckOut ].
	price := 0.
	aCart do: [:product | 
			price := price + priceList at: product.
		].
	(aCard isExpiredOn: date) ifTrue: [ ^self error: self class creditCardExpired ].
	merchantProcessor debit: price from: aCard.
	result := (merchantProcessor transactionsFor: aCard) last result.
	result ifTrue: [ 
		sale := Sale cashier: self card: aCard products: aCart priceList: priceList.
		sales add: sale.]
		! !

!Cashier methodsFor: 'creation' stamp: 'achavez 6/3/2013 20:17'!
date: aDate withPriceList: aPriceList merchantProcessor: aMerchantProcessor 
	date := aDate.
	priceList := aPriceList.
	merchantProcessor := aMerchantProcessor.
	sales := OrderedCollection new! !

!Cashier methodsFor: 'accessing' stamp: 'achavez 5/30/2013 20:56'!
lastSale
	^sales notEmpty ifTrue: [sales last]! !

Object subclass: #CreditCard
	instanceVariableNames: 'number owner expirationDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!CreditCard class methodsFor: 'instance creation' stamp: 'achavez 5/30/2013 20:10'!
number: aNumber owner: aString expirationDate: aDate 
	^self new number: aNumber owner: aString expirationDate: aDate ! !

!CreditCard methodsFor: 'accessing' stamp: 'achavez 6/5/2013 21:25'!
isExpiredOn: aDate
	^expirationDate < aDate! !

!CreditCard methodsFor: 'creation' stamp: 'achavez 5/30/2013 20:11'!
number: aNumber owner: aString expirationDate: aDate 
	number := aNumber.
	owner := aString.
	expirationDate := aDate.! !

Object subclass: #PlugableMerchantProcessor
	instanceVariableNames: 'action transactions'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!PlugableMerchantProcessor class methodsFor: 'error' stamp: 'achavez 6/5/2013 20:14'!
stolenCard
	^'The specified card has been reported stolen'! !

!PlugableMerchantProcessor class methodsFor: 'instance creation' stamp: 'achavez 6/9/2013 01:08'!
withAction: aBlock
	^self new withAction: aBlock! !

!PlugableMerchantProcessor methodsFor: 'services' stamp: 'achavez 6/9/2013 02:17'!
debit: anAmount from: aCard 
	|existing result transaction|
	existing := self transactionsFor: aCard.
	result := true.
	[
		[action value] on: Error do: [:e| result := false. e pass].
	] ensure: [
		transaction := Transaction amount: anAmount card: aCard result: result.
		existing add: transaction.
		transactions at: aCard put: existing.
		]! !

!PlugableMerchantProcessor methodsFor: 'accessing' stamp: 'achavez 6/5/2013 20:13'!
transactionsFor: aCard 
	^transactions at: aCard ifAbsent: OrderedCollection new! !

!PlugableMerchantProcessor methodsFor: 'creation' stamp: 'achavez 6/9/2013 01:16'!
withAction: aBlock
	action := aBlock.
	transactions := Dictionary new.! !

Object subclass: #RestInterface
	instanceVariableNames: 'catalog carts users time'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!RestInterface class methodsFor: 'error' stamp: 'achavez 6/5/2013 21:15'!
invalidPassword
	^'Specified user name and password are incorrect'! !

!RestInterface class methodsFor: 'error' stamp: 'achavez 6/5/2013 21:15'!
invalidUser
	^'Specified user name is invalid'! !

!RestInterface class methodsFor: 'error' stamp: 'achavez 6/8/2013 23:18'!
timeOutError
	^'Session time out'! !

!RestInterface class methodsFor: 'instance creation' stamp: 'achavez 6/8/2013 21:59'!
withCatalog: aCatalog clock: aTime
	^self new withCatalog: aCatalog clock: aTime! !

!RestInterface methodsFor: 'services' stamp: 'achavez 6/9/2013 01:30'!
add: aProduct quantity: aNumber to: aCart 
	|session|
	session := self cartSessionFor: aCart.
	(session isTimedOutAt: time) ifTrue: [ self error: self class timeOutError ].
	aNumber timesRepeat: [aCart add: aProduct]
	
	! !

!RestInterface methodsFor: 'accessing' stamp: 'achavez 6/9/2013 00:54'!
cartFor: aUserName
	| id |
	id := self idFor: aUserName.
	^(carts at: id ifAbsent: [^nil]) cart.! !

!RestInterface methodsFor: 'accessing' stamp: 'achavez 6/9/2013 01:04'!
cartFromId: aNumber
	|session|
	session := carts at: aNumber ifAbsent: [^nil].
	^session cart.
! !

!RestInterface methodsFor: 'accessing' stamp: 'achavez 6/8/2013 23:29'!
cartSessionFor: aCart
	carts keysAndValuesDo: [ :key :value | value cart = aCart ifTrue: [ ^value ] ].
	^nil! !

!RestInterface methodsFor: 'services' stamp: 'achavez 6/9/2013 00:54'!
createCartFor: aUserName pass: aPassword
	|valid cart session id|
	valid := self validate: aUserName pass: aPassword.
	valid ifTrue: [ 
		cart := Cart withCatalog: catalog. 
		id := self idFor: aUserName.
		session := CartSession for: cart id: id timestamp: time.
		carts at: id put: session.
		].
	^session! !

!RestInterface methodsFor: 'accessing' stamp: 'achavez 6/9/2013 01:03'!
idFor: aUserName
	|pass|
	pass := users at: aUserName ifAbsent: nil.
	^aUserName hash bitXor: pass hash.
! !

!RestInterface methodsFor: 'services' stamp: 'achavez 6/9/2013 01:32'!
listCart: aNumber 
	|session cart|
	session := self cartSessionFor: (self cartFromId: aNumber).
	(session isTimedOutAt: time) ifTrue: [ self error: self class timeOutError ].
	cart := (self cartFromId: aNumber) ifNil: [ ^#() ].
	^cart asArray! !

!RestInterface methodsFor: 'services' stamp: 'achavez 6/5/2013 21:13'!
validate: aUserName pass: aPassword
	|pass|
	pass := users at: aUserName ifAbsent: [ ^self error: self class invalidUser ].
	pass = aPassword ifFalse: [ ^self error: self class invalidPassword ].
	^true
	! !

!RestInterface methodsFor: 'creation' stamp: 'achavez 6/9/2013 02:26'!
withCatalog: aCatalog clock: aTime
	users := Dictionary new.
	carts := Dictionary new.
	catalog := aCatalog.
	time := aTime! !

Object subclass: #Sale
	instanceVariableNames: 'cashier card products priceList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Sale class methodsFor: 'instance creation' stamp: 'achavez 6/3/2013 20:09'!
cashier: aCashier card: aCard products: aCollection priceList: aPriceList
	^self new cashier: aCashier card: aCard products: aCollection priceList: aPriceList! !

!Sale methodsFor: 'accessing' stamp: 'achavez 6/9/2013 02:40'!
card
	^ card! !

!Sale methodsFor: 'accessing' stamp: 'achavez 6/9/2013 02:40'!
cashier
	^ cashier! !

!Sale methodsFor: 'creation' stamp: 'achavez 6/9/2013 01:18'!
cashier: aCashier card: aCard products: aCollection priceList: aPriceList 
	cashier := aCashier.
	card := aCard.
	products := aCollection.
	priceList := aPriceList! !

!Sale methodsFor: 'accessing' stamp: 'achavez 6/9/2013 02:41'!
priceList
	^ priceList! !

!Sale methodsFor: 'accessing' stamp: 'achavez 6/9/2013 02:41'!
products
	^ products! !

Object subclass: #Transaction
	instanceVariableNames: 'amount card result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Transaction class methodsFor: 'instance creation' stamp: 'achavez 6/3/2013 20:53'!
amount: anAmount card: aCard result: aResult 
	^self new amount: anAmount card: aCard result: aResult! !

!Transaction methodsFor: 'creation' stamp: 'achavez 6/3/2013 21:37'!
amount: anAmount card: aCard result: aResult 
	amount := anAmount.
	card := aCard.
	result := aResult ! !

!Transaction methodsFor: 'accessing' stamp: 'achavez 6/3/2013 21:02'!
result
	^ result! !

Bag subclass: #Cart
	instanceVariableNames: 'catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!Cart class methodsFor: 'error' stamp: 'msalegas 5/29/2013 20:07'!
productIsNotInTheCatalog
	^'Product not in the catalog'! !

!Cart class methodsFor: 'instance creation' stamp: 'achavez 5/30/2013 20:31'!
withCatalog: aCatalog 
	^self new catalog: aCatalog! !

!Cart methodsFor: 'services' stamp: 'msalegas 5/29/2013 20:37'!
add: aProduct
	(catalog includesKey: aProduct) ifTrue: [super add: aProduct] ifFalse: [ self error: self class productIsNotInTheCatalog]
	! !

!Cart methodsFor: 'accessing' stamp: 'msalegas 5/29/2013 19:33'!
catalog: aCatalog 
	catalog := aCatalog! !

TestCase subclass: #TestCart
	instanceVariableNames: 'timer interface catalog book user pass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'TusLibros'!

!TestCart methodsFor: 'setup' stamp: 'achavez 6/9/2013 01:08'!
setUp	
	book := 10.
	catalog := Dictionary new.
	catalog at: book put: 30.
	timer := Time now.
	interface := RestInterface withCatalog: catalog clock: timer.
	user := 'Pepe'.
	pass := 'xxx'.
	(interface instVarNamed: 'users') at: user put: pass.
	
	! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 6/9/2013 01:20'!
test01CartShouldBeEmptyWhenCreated
	|cart|
	cart := Cart withCatalog: catalog.
	self assert: cart isEmpty! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 6/9/2013 01:21'!
test02CartShouldListBooksWhenABookIsAdded
	|cart |
	cart := Cart withCatalog: catalog.
	cart add: book.
	self assert: (cart includes: book); assert: cart size = 1! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 6/9/2013 01:21'!
test03ABookShouldNotBeAddedToCartIfItIsNotInTheCatalog
	|cart|
	catalog := Dictionary new.
	cart := Cart withCatalog: catalog.
	self should: [cart add: book] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = Cart productIsNotInTheCatalog ]. 
	self assert: cart isEmpty
	! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 6/9/2013 01:22'!
test04AddABookMultipleTimesMakesTheCartSizeToIncrease
	|cart|
	cart := Cart withCatalog: catalog.	
	cart add: book; add: book.
	self assert: (cart size = 2)
	! !

!TestCart methodsFor: '1 - Cart' stamp: 'achavez 6/9/2013 01:24'!
test05CartShouldIncludeAllBooksWhenMultipleBooksAreAdded
	|anotherBook cart|
	anotherBook := 22.
	catalog at: anotherBook put: 10.
	cart := Cart withCatalog: catalog.
	cart add: book; add: anotherBook.
	self assert: (cart includes: book); 
		assert: (cart includes: anotherBook);
		assert: cart size = 2.! !

!TestCart methodsFor: '2 - Cashier' stamp: 'achavez 6/9/2013 02:24'!
test06CheckOutWithEmptyCartRaisesErrorAndNoSaleIsDone
	|cart merchantProcessor cashier card sale|
	cart := Cart withCatalog: Set new.
	merchantProcessor := PlugableMerchantProcessor new.
	cashier := Cashier date: Date today withPriceList: Set new merchantProcessor: merchantProcessor.
	card := CreditCard new.
	self should: [cashier checkout: cart card: card] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = Cashier invalidCheckOut ].
	sale := cashier lastSale.
	self assert: sale isNil.
	! !

!TestCart methodsFor: '2 - Cashier' stamp: 'achavez 6/9/2013 02:24'!
test07CheckOutWithExpiredCreditCardMakesNoSaleAndRaisesErrorAndDoesNotUseMerchantProcessorServices
	|cart merchantProcessor cashier card |
	cart := Cart withCatalog: catalog.
	cart add: book.
	merchantProcessor := PlugableMerchantProcessor withAction: [].
	cashier := Cashier date: Date today withPriceList: catalog merchantProcessor: merchantProcessor.
	card := CreditCard number: 0001000100010001 owner: 'Pedro Perez' expirationDate: Date today month - 1.
	self should: [ cashier checkout: cart card: card ] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = Cashier creditCardExpired ]. 
	self assert: cashier lastSale isNil;
		assert:  (merchantProcessor transactionsFor: cashier) isEmpty
	
	
	! !

!TestCart methodsFor: '2 - Cashier' stamp: 'achavez 6/9/2013 02:24'!
test08CheckOutWithStolenCreditCardMakeNoSaleAndMPTransactionFails
	|cart merchantProcessor cashier card sale|
	cart := Cart withCatalog: catalog.
	cart add: book.
	merchantProcessor := PlugableMerchantProcessor withAction: [self error: PlugableMerchantProcessor stolenCard].
	cashier := Cashier date: Date today withPriceList: catalog merchantProcessor: merchantProcessor.
	card := CreditCard number: 0001000100010001 owner: 'Pedro Perez' expirationDate: Date today month next.
	self should: [cashier checkout: cart card: card] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = PlugableMerchantProcessor stolenCard ].
	sale := cashier lastSale.
	self assert: sale isNil;
		deny: (merchantProcessor transactionsFor: card) anyOne result
	
		
	
	! !

!TestCart methodsFor: '2 - Cashier' stamp: 'achavez 6/9/2013 02:40'!
test09CheckOutWithCreditCardWithMakesASale
	|cart merchantProcessor cashier card sale|
	cart := Cart withCatalog: catalog.
	cart add: book.
	merchantProcessor := PlugableMerchantProcessor withAction: [].
	cashier := Cashier date: Date today withPriceList: catalog merchantProcessor: merchantProcessor.
	card := CreditCard number: 0001000100010001 owner: 'Pedro Perez' expirationDate: Date today month next.
	cashier checkout: cart card: card.
	sale := cashier lastSale.
	self deny: sale isNil;
		assert: (merchantProcessor transactionsFor: card) anyOne result;
		assert: sale cashier = cashier;
		assert: sale card = card;
		assert: sale priceList = catalog;
		assert: sale products = cart 
		
	
	! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/9/2013 02:31'!
test10CreateACartWithInvalidUserNameRaisesErrorAndDoesNotCreateACart
	|cart anotherUser|
	anotherUser := 'Pancho'.
	self assert: anotherUser ~= user.
	self should: [ cart := interface createCartFor: anotherUser pass: pass ] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = RestInterface invalidUser ].
	self assert: cart isNil; 
		assert: (interface cartFor: user) isNil! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/9/2013 01:38'!
test11CreateACartWithInvalidPasswordRaisesErrorAndDoesNotCreateACart
	|cart|
	self should: [ cart := interface createCartFor: user pass: pass, 'x' ] raise: Error withExceptionDo: [ :anError | self assert: anError messageText = RestInterface invalidPassword ].
	self assert: cart isNil;
		assert: (interface cartFor: user) isNil! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/8/2013 23:13'!
test12CreateACartWithValidUserAndPasswordCreatesACart 
	|session cart|
	session := interface createCartFor: user pass: pass.	
	cart := session cart.
	self deny: cart isNil;
		deny: (interface cartFor: user) isNil;
		assert: cart = (interface cartFor: user);
		assert: cart isEmpty! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/9/2013 00:07'!
test13AddABookToCartWithing30MinutesAddTheBookIndeed
	|session cart|
	session := interface createCartFor: user pass: pass.	
	cart := session cart.
	interface add: book quantity: 1 to: cart.
	self deny: cart isEmpty;
		assert: (cart includes: book)! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/9/2013 01:34'!
test14AddABookToCartAfter30MinuteSessionFailsAndDoesNotAddTheBook
	|session thirtyMinutesFromNow cart|
	session := interface createCartFor: user pass: pass.	
	thirtyMinutesFromNow := (interface instVarNamed: 'time') addSeconds: CartSession timeOutInSeconds+1.
	interface instVarNamed: 'time' put: thirtyMinutesFromNow.
	cart := session cart.
	self should: [interface add: book quantity: 1 to: cart] raise: Error withExceptionDo: [ :anError| self assert: anError messageText = interface class timeOutError ].
	self assert: cart isEmpty ! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/9/2013 00:32'!
test15AddABookToCartAndListWithin30MinuteSessionListsCorrectly
	|session cart|
	session := interface createCartFor: user pass: pass.	
	cart := session cart.
	interface add: book quantity: 10 to: cart.
	self assert: ((interface listCart: session id) includes: book)! !

!TestCart methodsFor: '3 - Client' stamp: 'achavez 6/9/2013 01:34'!
test16AddABookToCartAndListAfter30MinuteSessionFailsAndDoesNotListsCorrectly
	|session cart thirtyMinutesFromNow|
	session := interface createCartFor: user pass: pass.	
	cart := session cart.
	interface add: book quantity: 10 to: cart.
	thirtyMinutesFromNow := (interface instVarNamed: 'time') addSeconds: CartSession timeOutInSeconds+1.
	interface instVarNamed: 'time' put: thirtyMinutesFromNow.	
	self should: [interface listCart: session id] raise: Error withExceptionDo: [ :anError| self assert: anError messageText = interface class timeOutError ].
! !
